@@ -1899,8 +1899,8 @@ static void cmd_lqueue(char *arg)
 		struct tree_track *t;
 		struct rb_node *tmp;
  
-		rb_for_each_entry(t, tmp, &album->track_root, tree_node)
-			editable_add(&pq_editable, simple_track_new(tree_track_info(t)));
+		rb_for_each_entry(t, tmp, &a->album->track_root, tree_node)
+			play_queue_append(tree_track_info(t));
 		free(a);
 		item = next;
 	} while (item != &head);
@@ -1908,6 +1908,11 @@ unlock:
 	editable_unlock();
 }
 
+struct track_list {
+	struct list_head node;
+	const struct simple_track *track;
+};
+
 static void cmd_tqueue(char *arg)
 {
 	LIST_HEAD(head);
@@ -1934,13 +1939,14 @@ static void cmd_tqueue(char *arg)
 	item = lib_editable.head.next;
 	pos = 0;
 	for (i = 0; i < count; i++) {
-		struct simple_track *t;
+		struct track_list *t;
 
 		while (pos < r[i]) {
 			item = item->next;
 			pos++;
 		}
-		t = simple_track_new(to_simple_track(item)->info);
+		t = xnew(struct track_list, 1);
+		t->track = to_simple_track(item);
 		list_add_rand(&head, &t->node, i);
 	}
 	free(r);
@@ -1948,8 +1954,9 @@ static void cmd_tqueue(char *arg)
 	item = head.next;
 	do {
 		struct list_head *next = item->next;
-		struct simple_track *t = to_simple_track(item);
-		editable_add(&pq_editable, t);
+		struct track_list *t = container_of(item, struct track_list, node);
+		play_queue_append(t->track->info);
+		free(t);
 		item = next;
 	} while (item != &head);
 unlock:
