@@ -10,6 +10,7 @@
 #include "options.h"
 #include "debug.h"
 #include "utils.h"
+#include "file.h"
 
 #include <unistd.h>
 #include <inttypes.h>
@@ -472,23 +473,6 @@ static int v2_4_0_frame_header_parse(struct v2_frame_header *header, const char
 	return 1;
 }
 
-static int read_all(int fd, char *buf, size_t size)
-{
-	size_t pos = 0;
-
-	while (pos < size) {
-		int rc = read(fd, buf + pos, size - pos);
-
-		if (rc == -1) {
-			if (errno == EINTR || errno == EAGAIN)
-				continue;
-			return -1;
-		}
-		pos += rc;
-	}
-	return 0;
-}
-
 static char *parse_genre(const char *str)
 {
 	int parenthesis = 0;
@@ -1004,7 +988,7 @@ static int v2_read(struct id3tag *id3, int fd, const struct v2_header *header)
 	buf_size = header->size;
 	buf = xnew(char, buf_size);
 	rc = read_all(fd, buf, buf_size);
-	if (rc) {
+	if (rc == -1) {
 		free(buf);
 		return rc;
 	}
@@ -1118,7 +1102,7 @@ int id3_read_tags(struct id3tag *id3, int fd, unsigned int flags)
 		char buf[138];
 
 		rc = read_all(fd, buf, 10);
-		if (rc)
+		if (rc == -1)
 			goto rc_error;
 		if (v2_header_parse(&header, buf)) {
 			rc = v2_read(id3, fd, &header);
@@ -1132,7 +1116,7 @@ int id3_read_tags(struct id3tag *id3, int fd, unsigned int flags)
 			if (off == -1)
 				goto error;
 			rc = read_all(fd, buf, 138);
-			if (rc)
+			if (rc == -1)
 				goto rc_error;
 
 			if (is_v1(buf + 10)) {
@@ -1142,7 +1126,7 @@ int id3_read_tags(struct id3tag *id3, int fd, unsigned int flags)
 				}
 				if (v2_footer_parse(&header, buf)) {
 					/* footer at end of file - 128 */
-					off = lseek(fd, -(header.size + 138), SEEK_END);
+					off = lseek(fd, -((off_t) header.size + 138), SEEK_END);
 					if (off == -1)
 						goto error;
 					rc = v2_read(id3, fd, &header);
@@ -1151,7 +1135,7 @@ int id3_read_tags(struct id3tag *id3, int fd, unsigned int flags)
 				}
 			} else if (v2_footer_parse(&header, buf + 128)) {
 				/* footer at end of file */
-				off = lseek(fd, -(header.size + 10), SEEK_END);
+				off = lseek(fd, -((off_t) header.size + 10), SEEK_END);
 				if (off == -1)
 					goto error;
 				rc = v2_read(id3, fd, &header);
@@ -1166,7 +1150,7 @@ int id3_read_tags(struct id3tag *id3, int fd, unsigned int flags)
 		if (off == -1)
 			goto error;
 		rc = read_all(fd, id3->v1, 128);
-		if (rc)
+		if (rc == -1)
 			goto rc_error;
 		id3->has_v1 = is_v1(id3->v1);
 	}
